/*
 * James's header file for the Sound Galaxy cards
 */

/* $Id: jy_sg.h,v 1.4 1996/03/26 22:51:30 james Exp james $ */

/*
 * $Log: jy_sg.h,v $
 * Revision 1.4  1996/03/26 22:51:30  james
 * Don't declare the symbols for the meanings of the bits in the
 * configuration byte; instead, supply the prototypes for functions that
 * encode and decode the configuratino byte's value, along with the
 * definition of the structure it's exploded into.
 *
 * Revision 1.3  1996/03/25 02:18:02  james
 * Added config byte field bits
 *
 * Revision 1.2  1995/03/31 05:50:36  james
 * Added EEPROM write command numbers and support for EEPROM block
 * reads/writes.
 *
 * Revision 1.1  1995/03/30  20:23:15  james
 * Initial revision
 *
 */

typedef unsigned char BYTE;
typedef unsigned short IOADDR;

extern IOADDR Bbase;		/* SoundBlaster-compatible DSP base address */

/*
 * DSP I/O ports.
 *
 * Note that DSP_COMMAND_PORT is the same as DSP_WRITE_STATUS_PORT.
 */
#define DSP_RESET_PORT		(Bbase+0x06)
#define DSP_DATA_PORT		(Bbase+0x0A)
#define DSP_COMMAND_PORT	(Bbase+0x0C) /* write */
#define DSP_WRITE_STATUS_PORT	(Bbase+0x0C) /* read */
#define DSP_DATA_AVAILABLE_PORT	(Bbase+0x0E)

/*
 * DSP values
 */
#define DSP_WRITE_BUSY_BIT (1<<7)
#define DSP_DATA_AVAILABLE_BIT (1<<7)


/*
 * EEPROM functions
 */
#define EEPROM_READ_COMMAND 0x08
#define EEPROM_READ_SUBCOMMAND 0x07

#define EEPROM_WRITE_COMMAND 0x08
#define EEPROM_WRITE_SUBCOMMAND 0x08


/*
 * EEPROM sizes in bytes.
 * Note that the maximum EEPROM address is therefore one less than this number.
 */				
#define SG_NX_II_EEPROM_SIZE	(0x01) /* SG NX II */
#define SG_NX_PRO_EEPROM_SIZE	(0x0D) /* SG NX Pro and SG NX Pro 16 */
#define SG_PRO_16_EEPROM_SIZE	(0x24) /* SG Pro 16 */
#define SG_NOVA_16_EEPROM_SIZE	(0x10) /* SG Nova 16 */
#define SG_LYRA_16_EEPROM_SIZE	(0x10) /* SG Lyra 16 */
#define MAX_SG_EEPROM_SIZE 	SG_PRO_16_EEPROM_SIZE


/*
 * Prototypes.
 */

/*
 * Generic port IO functions.
 */
void write_dsp(BYTE val);
BYTE read_dsp(void);
void set_sb_bbase(IOADDR base_addr);
IOADDR get_sb_bbase(void);
int  detect_dsp(void);
int open_port_device(const char *name);
enum SgStatus close_port_device(void);
void write_io_byte(IOADDR addr, BYTE val);
BYTE read_io_byte(IOADDR addr);
BYTE read_eeprom(BYTE ee_loc);
void read_eeprom_block(BYTE* data, BYTE n);
void write_eeprom_block(BYTE* data, BYTE n);

struct sgnx_eeprom_config
{
  int cd_is_atbus;
  int cdrom_addr;
  int game_port_enabled;
  int dsp_and_mixer_addr;
  int dma;
  int irq;
};

int decode_sgnx_config_byte(BYTE val, struct sgnx_eeprom_config *p);
int encode_sgnx_config_byte(const struct sgnx_eeprom_config *p, BYTE *pb);
const char * decode_sgnx_config(const struct sgnx_eeprom_config *p);


enum SgInitResult
{
  SG_INIT_OK,
};
/* Pass the Sound Galaxy base address to this function, or 0 to
 * attempt autodetection.  You may also want to pass the name of
 * the IO-port device, which defaults to /dev/port.  Pass NULL
 * to select the default for this parameter.
 */
enum SgStatus
{
  SG_OK, SG_ERROR
};

enum SgStatus sg_init(IOADDR sg_addr, const char *port_device);


enum SgError 
{
  SG_ERR_NONE,
  SG_ERR_OPEN_PORT_DEVICE,
  SG_ERR_DETECT_DSP
};


/* Error detection
 */
enum SgError sg_get_error(void);
int sg_is_errno_interesting(void);
int sg_get_error_errno_value(void);
void sg_clear_error(void);
const char *sg_get_error_message(void);

/* Setting error information.  Don't use these....
 */
void sg_set_error(enum SgError err);
void sg_set_syscall_error(int myerrno);
